searchData={"items":[{"type":"module","title":"AyeSQL","doc":"_AyeSQL_ is a library for using raw SQL.\n\n> **Aye** _/ʌɪ/_ _exclamation (archaic dialect)_: said to express assent; yes.","ref":"AyeSQL.html"},{"type":"module","title":"Overview - AyeSQL","doc":"Inspired by Clojure library [Yesql](https://github.com/krisajenkins/yesql),\n_AyeSQL_ tries to find a middle ground between strings with raw SQL queries and\nSQL DSLs by:\n\n- Keeping SQL in SQL files.\n- Generating Elixir functions for every query.\n- Supporting mandatory and optional named parameters.\n- Allowing query composability with ease.\n- Working out of the box with PostgreSQL using\n  [Ecto](https://github.com/elixir-ecto/ecto_sql) or\n  [Postgrex](https://github.com/elixir-ecto/postgrex):\n- Being extended to support other databases using the behaviour\n  `AyeSQL.Runner`.","ref":"AyeSQL.html#module-overview"},{"type":"module","title":"Small Example - AyeSQL","doc":"Let's say we have a\n[SQL query](https://stackoverflow.com/questions/39556763/use-ecto-to-generate-series-in-postgres-and-also-retrieve-null-values-as-0)\nto retrieve the click count of a certain type of link every day of the last `X`\ndays. In raw SQL this could be written as:\n\n```sql\n    WITH computed_dates AS (\n           SELECT dates::date AS date\n             FROM generate_series(\n                    current_date - $1::interval,\n                    current_date - interval '1 day',\n                    interval '1 day'\n                  ) AS dates\n         )\n  SELECT dates.date AS day, count(clicks.id) AS count\n    FROM computed_dates AS dates\n         LEFT JOIN clicks AS clicks ON date(clicks.inserted_at) = dates.date\n   WHERE clicks.link_id = $2\nGROUP BY dates.date\nORDER BY dates.date;\n```\n\nThe equivalent query in Ecto would be:\n\n```elixir\ndates = ~s(\nSELECT generate_series(\n         current_date - ?::interval,\n         current_date - interval '1 day',\n         interval '1 day'\n       )::date AS d\n)\n\nfrom(\n  c in \"clicks\",\n  right_join: day in fragment(dates, ^days),\n  on: day.d == fragment(\"date(?)\", c.inserted_at),\n  where: c.link_id = ^link_id\n  group_by: day.d,\n  order_by: day.d,\n  select: %{\n    day: fragment(\"date(?)\", day.d),\n    count: count(c.id)\n  }\n)\n```\n\nUsing fragments can get convoluted and difficult to maintain. In AyeSQL, the\nequivalent would be to create an SQL file with the query e.g. `queries.sql`:\n\n```sql\n-- name: get_day_interval\nSELECT datetime::date AS date\n  FROM generate_series(\n        current_date - :days::interval, -- Named parameter :days\n        current_date - interval '1 day',\n        interval '1 day'\n      );\n\n-- name: get_avg_clicks\n-- docs: Gets average click count.\n    WITH computed_dates AS ( :get_day_interval ) -- Composing with another query\n  SELECT dates.date AS day, count(clicks.id) AS count\n    FROM computed_date AS dates\n        LEFT JOIN clicks AS clicks ON date(clicks.inserted_at) = dates.date\n  WHERE clicks.link_id = :link_id -- Named parameter :link_id\nGROUP BY dates.date\nORDER BY dates.date;\n```\n\nIn Elixir, we would load all the queries in this file by creating the following\nmodule:\n\n```elixir\ndefmodule Queries do\n  use AyeSQL, repo: MyRepo\n\n  defqueries(\"queries.sql\") # File name with relative path to SQL file.\nend\n```\n\nor using the macro `defqueries/3`:\n\n```elixir\nimport AyeSQL, only: [defqueries: 3]\n\ndefqueries(Queries, \"queries.sql\", repo: MyRepo)\n```\n\nBoth approaches will create a module called `Queries` with all the queries\ndefined in `queries.sql`.\n\nAnd then we could execute the query as follows:\n\n```elixir\niex> params = [\n...>   link_id: 42,\n...>   days: %Postgrex.Interval{secs: 864_000} # 10 days\n...> ]\niex> Queries.get_avg_clicks(params)\n{:ok,\n  [\n    %{day: ..., count: ...},\n    %{day: ..., count: ...},\n    %{day: ..., count: ...},\n    ...\n  ]\n}\n```\nAyeSQL also allows you to choose the type of returned data structures.\nInstead of the default map you can also pass an `into` option to your query\npossible values are:\n- an empty map: `Map.new()` or `%{}`\n- an empty list: `Keyword.new()` or `[]`\n- a struct\n- `:raw` which returns the unmodified Postgrex result\n\n```elixir\niex> Queries.get_avg_clicks(params, into: [])\n{:ok,\n  [\n    [day: ..., count: ...],\n    [day: ..., count: ...],\n    [day: ..., count: ...],\n    ...\n  ]\n}\n```\n\n```elixir\niex> defmodule AvgClicks do defstruct [:day, :count] end\niex> Queries.get_avg_clicks(params, into: AvgClicks)\n{:ok,\n  [\n    %AvgClicks{day: ..., count: ...},\n    %AvgClicks{day: ..., count: ...},\n    %AvgClicks{day: ..., count: ...},\n    ...\n  ]\n}\n```","ref":"AyeSQL.html#module-small-example"},{"type":"macro","title":"AyeSQL.__using__/1","doc":"Uses `AyeSQL` for loading queries.\n\nBy default, supports the option `runner` (see `AyeSQL.Runner` behaviour).\n\nAny other option will be passed to the runner.","ref":"AyeSQL.html#__using__/1"},{"type":"macro","title":"AyeSQL.defqueries/1","doc":"Macro to load queries from a `file`.\n\nLet's say we have the file `lib/sql/queries.sql` with the following contents:\n\n```sql\n-- name: get_user\n-- docs: Gets user by username\nSELECT *\n  FROM users\n WHERE username = :username;\n```\n\nThen we can load our queries to Elixir using the macro `defqueries/1`:\n\n```\n# file: lib/queries.ex\ndefmodule Queries do\n  use AyeSQL, repo: MyRepo\n\n  defqueries(\"sql/queries.sql\")\nend\n```\n\nor the macro `defqueries/3`:\n\n```\n# file: lib/queries.ex\nimport AyeSQL, only: [defqueries: 3]\n\ndefqueries(Queries, \"sql/queries.ex\", repo: MyRepo)\n```\n\nAnd finally we can inspect the query:\n\n```\niex(1)> Queries.get_user(username: \"some_user\", run: false)\n{:ok,\n  %AyeSQL.Query{\n    statement: \"SELECT * FROM user WHERE username = $1\",\n    arguments: [\"some_user\"]\n  }\n}\n```\n\nor run it:\n\n```\niex(1)> Queries.get_user(username: \"some_user\")\n{:ok,\n  [\n    %{username: ..., ...}\n  ]\n}\n```","ref":"AyeSQL.html#defqueries/1"},{"type":"macro","title":"AyeSQL.defqueries/3","doc":"Macro to load queries from a `file` and create a module for them.\n\nSame as `defqueries/1`, but creates a module e.g for the query file\n`lib/sql/queries.sql` we can use this macro as follows:\n\n```\n# file: lib/queries.ex\nimport AyeSQL, only: [defqueries: 3]\n\ndefqueries(Queries, \"sql/queries.sql\", repo: MyRepo)\n```\n\nThis will generate the module `Queries` and it'll contain all the SQL\nstatements included in `sql/queries.sql`.","ref":"AyeSQL.html#defqueries/3"},{"type":"function","title":"AyeSQL.eval_query/2","doc":"Evaluates the `contents` of a string with a query and generates an anonyous\nfunction that receives parameters and options.","ref":"AyeSQL.html#eval_query/2"},{"type":"module","title":"AyeSQL.Core","doc":"This module defines the core functionality for AyeSQL.","ref":"AyeSQL.Core.html"},{"type":"type","title":"AyeSQL.Core.options/0","doc":"AyeSQL query options.","ref":"AyeSQL.Core.html#t:options/0"},{"type":"type","title":"AyeSQL.Core.parameter_name/0","doc":"AyeSQL query parameter name.","ref":"AyeSQL.Core.html#t:parameter_name/0"},{"type":"type","title":"AyeSQL.Core.parameters/0","doc":"AyeSQL query parameters.","ref":"AyeSQL.Core.html#t:parameters/0"},{"type":"module","title":"AyeSQL.Error","doc":"This module defines an AyeSQL error.","ref":"AyeSQL.Error.html"},{"type":"function","title":"AyeSQL.Error.__struct__/0","doc":"A query struct.","ref":"AyeSQL.Error.html#__struct__/0"},{"type":"function","title":"AyeSQL.Error.new/1","doc":"Creates a new error given some `options`.","ref":"AyeSQL.Error.html#new/1"},{"type":"type","title":"AyeSQL.Error.arguments/0","doc":"Query arguments.","ref":"AyeSQL.Error.html#t:arguments/0"},{"type":"type","title":"AyeSQL.Error.errors/0","doc":"Query errors.","ref":"AyeSQL.Error.html#t:errors/0"},{"type":"type","title":"AyeSQL.Error.statement/0","doc":"Query statement.","ref":"AyeSQL.Error.html#t:statement/0"},{"type":"type","title":"AyeSQL.Error.t/0","doc":"An error type.","ref":"AyeSQL.Error.html#t:t/0"},{"type":"module","title":"AyeSQL.Query","doc":"This module defines an AyeSQL query.","ref":"AyeSQL.Query.html"},{"type":"function","title":"AyeSQL.Query.__struct__/0","doc":"A query struct.","ref":"AyeSQL.Query.html#__struct__/0"},{"type":"function","title":"AyeSQL.Query.new/1","doc":"Creates a new query given some `options`.","ref":"AyeSQL.Query.html#new/1"},{"type":"type","title":"AyeSQL.Query.arguments/0","doc":"Query arguments.","ref":"AyeSQL.Query.html#t:arguments/0"},{"type":"type","title":"AyeSQL.Query.statement/0","doc":"Query statement.","ref":"AyeSQL.Query.html#t:statement/0"},{"type":"type","title":"AyeSQL.Query.t/0","doc":"A query type.","ref":"AyeSQL.Query.html#t:t/0"},{"type":"module","title":"AyeSQL.AST","doc":"This module defines function for expanding the AST.","ref":"AyeSQL.AST.html"},{"type":"type","title":"AyeSQL.AST.expand_function/0","doc":"Function to be applied with some parameters and context.","ref":"AyeSQL.AST.html#t:expand_function/0"},{"type":"module","title":"AyeSQL.AST.Context","doc":"This module defines an AST context.","ref":"AyeSQL.AST.Context.html"},{"type":"function","title":"AyeSQL.AST.Context.__struct__/0","doc":"AST context struct.","ref":"AyeSQL.AST.Context.html#__struct__/0"},{"type":"function","title":"AyeSQL.AST.Context.add_index/2","doc":"Adds a `value` to the `context` index.","ref":"AyeSQL.AST.Context.html#add_index/2"},{"type":"function","title":"AyeSQL.AST.Context.id/1","doc":"Context id function.","ref":"AyeSQL.AST.Context.html#id/1"},{"type":"function","title":"AyeSQL.AST.Context.merge/2","doc":"Merges two contexts.","ref":"AyeSQL.AST.Context.html#merge/2"},{"type":"function","title":"AyeSQL.AST.Context.merge_error/2","doc":"Merges a `context` with an `error`","ref":"AyeSQL.AST.Context.html#merge_error/2"},{"type":"function","title":"AyeSQL.AST.Context.merge_query/2","doc":"Merges a `context` with a `query`.","ref":"AyeSQL.AST.Context.html#merge_query/2"},{"type":"function","title":"AyeSQL.AST.Context.new/1","doc":"Creates a new context given some `options`.","ref":"AyeSQL.AST.Context.html#new/1"},{"type":"function","title":"AyeSQL.AST.Context.not_found/2","doc":"Updates `context` with the error not found for a `key`.","ref":"AyeSQL.AST.Context.html#not_found/2"},{"type":"function","title":"AyeSQL.AST.Context.put_argument/2","doc":"Adds arguments in a `context` given a new `value`.","ref":"AyeSQL.AST.Context.html#put_argument/2"},{"type":"function","title":"AyeSQL.AST.Context.put_statement/2","doc":"Adds statement in a `context` given a new `value`.","ref":"AyeSQL.AST.Context.html#put_statement/2"},{"type":"function","title":"AyeSQL.AST.Context.put_variable/2","doc":"Puts a new variable `value` in the `context`.","ref":"AyeSQL.AST.Context.html#put_variable/2"},{"type":"function","title":"AyeSQL.AST.Context.put_variables/2","doc":"Puts several variable `value` in the `context` as an SQL list.","ref":"AyeSQL.AST.Context.html#put_variables/2"},{"type":"function","title":"AyeSQL.AST.Context.to_query/1","doc":"Transforms a context to a query.","ref":"AyeSQL.AST.Context.html#to_query/1"},{"type":"type","title":"AyeSQL.AST.Context.arguments/0","doc":"Argument list.","ref":"AyeSQL.AST.Context.html#t:arguments/0"},{"type":"type","title":"AyeSQL.AST.Context.error/0","doc":"Error.","ref":"AyeSQL.AST.Context.html#t:error/0"},{"type":"type","title":"AyeSQL.AST.Context.error_type/0","doc":"Error type.","ref":"AyeSQL.AST.Context.html#t:error_type/0"},{"type":"type","title":"AyeSQL.AST.Context.index/0","doc":"Current context index.","ref":"AyeSQL.AST.Context.html#t:index/0"},{"type":"type","title":"AyeSQL.AST.Context.statement/0","doc":"Accumulated statement.","ref":"AyeSQL.AST.Context.html#t:statement/0"},{"type":"type","title":"AyeSQL.AST.Context.t/0","doc":"AST context.","ref":"AyeSQL.AST.Context.html#t:t/0"},{"type":"module","title":"AyeSQL.Compiler","doc":"This module defines functions to compile `AyeSQL` language strings.","ref":"AyeSQL.Compiler.html"},{"type":"function","title":"AyeSQL.Compiler.compile_queries/2","doc":"Compiles the `contents` of a file or string into valid AyeSQL queries.","ref":"AyeSQL.Compiler.html#compile_queries/2"},{"type":"function","title":"AyeSQL.Compiler.compile_query/2","doc":"Compiles a single query from the `contents` of a string.","ref":"AyeSQL.Compiler.html#compile_query/2"},{"type":"function","title":"AyeSQL.Compiler.eval_query/2","doc":"Evaluates the `contents` of a string to an anonymous function with a query\nthat receives parameters and options.","ref":"AyeSQL.Compiler.html#eval_query/2"},{"type":"type","title":"AyeSQL.Compiler.docs/0","doc":"Query documentation.","ref":"AyeSQL.Compiler.html#t:docs/0"},{"type":"type","title":"AyeSQL.Compiler.fragment/0","doc":"Query fragment.","ref":"AyeSQL.Compiler.html#t:fragment/0"},{"type":"type","title":"AyeSQL.Compiler.fragments/0","doc":"Query fragments.","ref":"AyeSQL.Compiler.html#t:fragments/0"},{"type":"type","title":"AyeSQL.Compiler.name/0","doc":"Query name.","ref":"AyeSQL.Compiler.html#t:name/0"},{"type":"type","title":"AyeSQL.Compiler.param/0","doc":"Query parameter.","ref":"AyeSQL.Compiler.html#t:param/0"},{"type":"type","title":"AyeSQL.Compiler.queries/0","doc":"Queries.","ref":"AyeSQL.Compiler.html#t:queries/0"},{"type":"type","title":"AyeSQL.Compiler.query/0","doc":"Query.","ref":"AyeSQL.Compiler.html#t:query/0"},{"type":"module","title":"AyeSQL.Lexer","doc":"This module defines the lexer for the `AyeSQL` language.","ref":"AyeSQL.Lexer.html"},{"type":"function","title":"AyeSQL.Lexer.tokenize/2","doc":"Gets tokens from the `contents` of a string.","ref":"AyeSQL.Lexer.html#tokenize/2"},{"type":"type","title":"AyeSQL.Lexer.column/0","doc":"Column number.","ref":"AyeSQL.Lexer.html#t:column/0"},{"type":"type","title":"AyeSQL.Lexer.line/0","doc":"Line number.","ref":"AyeSQL.Lexer.html#t:line/0"},{"type":"type","title":"AyeSQL.Lexer.location/0","doc":"Location.","ref":"AyeSQL.Lexer.html#t:location/0"},{"type":"type","title":"AyeSQL.Lexer.option/0","doc":"Lexer option.","ref":"AyeSQL.Lexer.html#t:option/0"},{"type":"type","title":"AyeSQL.Lexer.options/0","doc":"Lexer options.","ref":"AyeSQL.Lexer.html#t:options/0"},{"type":"type","title":"AyeSQL.Lexer.original/0","doc":"Original match from the string.","ref":"AyeSQL.Lexer.html#t:original/0"},{"type":"type","title":"AyeSQL.Lexer.token/0","doc":"Token.","ref":"AyeSQL.Lexer.html#t:token/0"},{"type":"type","title":"AyeSQL.Lexer.token_name/0","doc":"Token name.","ref":"AyeSQL.Lexer.html#t:token_name/0"},{"type":"type","title":"AyeSQL.Lexer.tokens/0","doc":"Tokens.","ref":"AyeSQL.Lexer.html#t:tokens/0"},{"type":"type","title":"AyeSQL.Lexer.value/0","doc":"Token value.","ref":"AyeSQL.Lexer.html#t:value/0"},{"type":"behaviour","title":"AyeSQL.Runner","doc":"This module defines an `AyeSQL.Runner`.","ref":"AyeSQL.Runner.html"},{"type":"macro","title":"AyeSQL.Runner.__using__/1","doc":"Uses the `AyeSQL.Runner` behaviour.","ref":"AyeSQL.Runner.html#__using__/1"},{"type":"callback","title":"AyeSQL.Runner.run/2","doc":"Callback to initialize the runner.","ref":"AyeSQL.Runner.html#c:run/2"},{"type":"module","title":"AyeSQL.Runner.Ecto","doc":"This module defines `Ecto` default adapter.\n\nCan be used as follows:\n\n```elixir\ndefmodule MyQueries do\n  use AyeSQL, repo: MyRepo\n\n  defqueries(\"query/my_queries.sql\")\nend\n```","ref":"AyeSQL.Runner.Ecto.html"},{"type":"module","title":"AyeSQL.Runner.Postgrex","doc":"This module defines `Postgrex` default adapter.\n\nCan be used as follows:\n\n```elixir\ndefmodule MyQueries do\n  use AyeSQL,\n    runner: AyeSQL.Runner.Postgrex\n\n  defqueries(\"query/my_queries.sql\")\nend\n```\n\nAnd given a `connection` to the database, then it can be used with the\nquery options:\n\n```elixir\niex> MyQueries.get_user([id: id], conn: connection)\n{:ok, ...}\n```","ref":"AyeSQL.Runner.Postgrex.html"},{"type":"exception","title":"AyeSQL.CompileError","doc":"This module defines an AyeSQL compile error exception.","ref":"AyeSQL.CompileError.html"},{"type":"function","title":"AyeSQL.CompileError.__struct__/0","doc":"A compiler error.","ref":"AyeSQL.CompileError.html#__struct__/0"},{"type":"type","title":"AyeSQL.CompileError.t/0","doc":"A compile error.","ref":"AyeSQL.CompileError.html#t:t/0"},{"type":"extras","title":"AyeSQL","doc":"# AyeSQL\n\n![Build status](https://github.com/alexdesousa/ayesql/actions/workflows/checks.yml/badge.svg) [![Hex pm](http://img.shields.io/hexpm/v/ayesql.svg?style=flat)](https://hex.pm/packages/ayesql) [![hex.pm downloads](https://img.shields.io/hexpm/dt/ayesql.svg?style=flat)](https://hex.pm/packages/ayesql) [![Coverage Status](https://coveralls.io/repos/github/alexdesousa/ayesql/badge.svg?branch=master)](https://coveralls.io/github/alexdesousa/ayesql?branch=master)\n\n> **Aye** _/ʌɪ/_ _exclamation (archaic dialect)_: said to express assent; yes.\n\n_AyeSQL_ is a library for using raw SQL.","ref":"readme.html"},{"type":"extras","title":"Overview - AyeSQL","doc":"Inspired by Clojure library [Yesql](https://github.com/krisajenkins/yesql),\n_AyeSQL_ tries to find a middle ground between strings with raw SQL queries and\nSQL DSLs. This library aims to:\n\n- Keep SQL in SQL files.\n- Generate easy to use Elixir functions for every query.\n- Parameterize queries using maps and keyword lists.\n- Allow query composablity.\n- Work out-of-the-box with PostgreSQL using\n  [Ecto](https://github.com/elixir-ecto/ecto_sql) or\n  [Postgrex](https://github.com/elixir-ecto/postgrex).\n\nIf you want to know more about AyeSQL:\n\n- [Small example](#small-example)\n- [Syntax](#syntax)\n\n  + [Naming queries](#naming-queries)\n  + [Parameters](#parameters)\n  + [Mandatory parameters](#mandatory-parameters)\n  + [Query composition](#query-composition)\n  + [Optional fragments](#optional-fragments)\n  + [IN statement](#in-statement)\n  + [Subqueries and subfragments](#subqueries-and-subfragments)\n\n- [Installation](#installation)\n\nAnd the following additional links provide more information about the library:\n\n- [Full Documentation](https://hexdocs.pm/ayesql)\n- [AyeSQL: Writing Raw SQL in Elixir](https://thebroken.link/ayesql-writing-raw-sql-in-elixir/)\n- [Why raw SQL?](https://hexdocs.pm/ayesql/why-raw-sql.html)\n- [Dynamic queries with EEx](https://hexdocs.pm/ayesql/dynamic-queries-with-eex.html)\n- [Adding support to other databases](https://hexdocs.pm/ayesql/query-runners.html)","ref":"readme.html#overview"},{"type":"extras","title":"Small Example - AyeSQL","doc":"In AyeSQL, the equivalent would be to create an SQL file with the query e.g.\n`queries.sql`:\n\n```sql\n-- file: queries.sql\n-- name: get_avg_clicks\n-- docs: Gets average click count.\n    WITH computed_dates AS (\n      SELECT datetime::date AS date\n      FROM generate_series(\n        current_date - :days::interval, -- Named parameter :days\n        current_date - interval '1 day',\n        interval '1 day'\n      )\n    )\n  SELECT dates.date AS day, count(clicks.id) AS count\n    FROM computed_date AS dates\n         LEFT JOIN clicks AS clicks ON date(clicks.inserted_at) = dates.date\n   WHERE clicks.link_id = :link_id -- Named parameter :link_id\nGROUP BY dates.date\nORDER BY dates.date;\n```\n\nIn Elixir, we would load all the queries in this file by creating the following\nmodule:\n\n```elixir\n# file: lib/queries.ex\ndefmodule Queries do\n  use AyeSQL, repo: MyRepo\n\n  defqueries(\"queries.sql\") # File name with relative path to SQL file.\nend\n```\n\nor using the macro `defqueries/3`:\n\n```elixir\n# file: lib/queries.ex\nimport AyeSQL, only: [defqueries: 3]\n\ndefqueries(Queries, \"queries.sql\", repo: MyRepo)\n```\n\n> **Note**: The file name used in `defqueries` macro should be relative to the\n> file where the macro is used.\n\nBoth approaches will create a module called `Queries` with all the queries\ndefined in `queries.sql`.\n\nAnd then we could execute the query as follows:\n\n```elixir\niex> params = [\n...>   link_id: 42,\n...>   days: %Postgrex.Interval{secs: 864_000} # 10 days\n...> ]\niex> Queries.get_avg_clicks(params)\n{:ok,\n  [\n    %{day: ..., count: ...},\n    %{day: ..., count: ...},\n    %{day: ..., count: ...},\n    ...\n  ]\n}\n```","ref":"readme.html#small-example"},{"type":"extras","title":"Syntax - AyeSQL","doc":"An SQL file can have as many queries as you want as long as they are named.\n\nFor the following sections we'll assume we have:\n\n- `lib/my_repo.ex` which is an `Ecto` repo called `MyRepo`.\n- `lib/queries.sql` with SQL queries.\n- `lib/queries.ex` with the following structure:\n\n    ```elixir\n    import AyeSQL, only: [defqueries: 3]\n\n    defqueries(Queries, \"queries.sql\", repo: MyRepo)\n    ```\n\n#","ref":"readme.html#syntax"},{"type":"extras","title":"Naming Queries - AyeSQL","doc":"For naming queries, we add a comment with the keyword `-- name: ` followed by\nthe name of the function e.g the following query would generate the function\n`Queries.get_hostnames/2`:\n\n```sql\n-- name: get_hostnames\nSELECT hostname FROM server\n```\n\nAdditionally, we could also add documentation for the query by adding a comment\nwith the keyword `-- docs: ` followed by the query's documentation e.g:\n\n```sql\n-- name: get_hostnames\n-- docs: Gets hostnames from the servers.\nSELECT hostname FROM server\n```\n\n> Important: if the function does not have `-- docs: ` it won't have\n> documentation e.g. `@doc false`.\n\n#","ref":"readme.html#naming-queries"},{"type":"extras","title":"Parameters - AyeSQL","doc":"There are two types of parameters:\n\n- Mandatory: for passing parameters to a query. They start with `:` e.g.\n  `:hostname`.\n- Optional: for query composability. They start with `:_` e.g. `:_order_by`.\n\nAdditionally, any query in a file can be accessed with its name adding `:` at\nthe front e.g `:get_hostnames`.\n\n#","ref":"readme.html#parameters"},{"type":"extras","title":"Mandatory Parameters - AyeSQL","doc":"Let's say we want to get the name of an operative system by architecture:\n\n```sql\n-- name: get_os_by_architecture\n-- docs: Gets operative system's name by a given architecture.\nSELECT name\n  FROM operative_system\n WHERE architecture = :architecture\n```\n\nThe previous query would generate the function\n`Queries.get_os_by_architecture/2` that can be called as:\n\n```elixir\niex> Queries.get_os_by_architecture(architecture: \"AMD64\")\n{:ok,\n  [\n    %{name: \"Debian Buster\"},\n    %{name: \"Windows 10\"},\n    ...\n  ]\n}\n```\n\n#","ref":"readme.html#mandatory-parameters"},{"type":"extras","title":"Query Composition - AyeSQL","doc":"Now if we would like to get hostnames by architecture we could compose queries\nby doing the following:\n\n```sql\n-- name: get_os_by_architecture\n-- docs: Gets operative system's name by a given architecture.\nSELECT name\n  FROM operative_system\n WHERE architecture = :architecture\n\n-- name: get_hostnames_by_architecture\n-- docs: Gets hostnames by architecture.\nSELECT hostname\n  FROM servers\n WHERE os_name IN ( :get_os_by_architecture )\n```\n\nThe previous query would generate the function\n`Queries.get_hostnames_by_architecture/2` that can be called as:\n\n```elixir\niex> Queries.get_hostnames_by_architecture(architecture: \"AMD64\")\n{:ok,\n  [\n    %{hostname: \"server0\"},\n    %{hostname: \"server1\"},\n    ...\n  ]\n}\n```\n\n#","ref":"readme.html#query-composition"},{"type":"extras","title":"Optional Fragments - AyeSQL","doc":"Let's say that now we need to order ascending or descending by hostname by\nusing an optional `:_order_by` parameter e.g:\n\n```sql\n-- name: get_os_by_architecture\n-- docs: Gets operative system's name by a given architecture.\nSELECT name\n  FROM operative_system\n WHERE architecture = :architecture\n\n-- name: get_hostnames_by_architecture\n-- docs: Gets hostnames by architecture.\nSELECT hostname\n  FROM servers\n WHERE os_name IN ( :get_os_by_architecture )\n :_order_by\n\n-- name: ascending\nORDER BY hostname ASC\n\n-- name: descending\nORDER BY hostname DESC\n```\n\nThe previous query could be called as before:\n\n```elixir\niex> Queries.get_hostnames_by_architecture(architecture: \"AMD64\")\n{:ok,\n  [\n    %{hostname: \"Barcelona\"},\n    %{hostname: \"Granada\"},\n    %{hostname: \"Madrid\"},\n    ...\n  ]\n}\n```\n\nor by order ascending:\n\n```elixir\niex> params = [architecture: \"AMD64\", _order_by: :ascending]\niex> Queries.get_hostnames_by_architecture(params)\n{:ok,\n  [\n    %{hostname: \"Barcelona\"},\n    %{hostname: \"Madrid\"},\n    %{hostname: \"Granada\"},\n    ...\n  ]\n}\n```\n\nor descending:\n\n```elixir\niex> params = [architecture: \"AMD64\", _order_by: :descending]\niex> Queries.get_hostnames_by_architecture(params)\n{:ok,\n  [\n    %{hostname: \"Zaragoza\"},\n    %{hostname: \"Madrid\"},\n    %{hostname: \"Granada\"},\n    ...\n  ]\n}\n```\n\n> Important: A query can be called by name e.g. `:descending` if it's defined\n> in the same SQL file. Otherwise, we need to pass the function instead e.g.\n> `Queries.descending/2`\n>\n> ```elixir\n> iex> params = [architecture: \"AMD64\", _order_by: Queries.descending/2]\n> iex> Queries.get_hostnames_by_architecture(params)\n> {:ok,\n>   [\n>     %{hostname: \"Zaragoza\"},\n>     %{hostname: \"Madrid\"},\n>     %{hostname: \"Granada\"},\n>     ...\n>   ]\n> }\n> ```\n\n#","ref":"readme.html#optional-fragments"},{"type":"extras","title":"IN Statement - AyeSQL","doc":"Lists in SQL might be tricky. That's why AyeSQL supports a special type for\nthem e.g:\n\nLet's say we have the following query:\n\n```sql\n-- name: get_os_by_hostname\n-- docs: Gets hostnames and OS names given a list of hostnames.\nSELECT hostname, os_name\n  FROM servers\n WHERE hostname IN (:hostnames)\n```\n\nIt is possible to do the following:\n\n```elixir\niex> params = [hostnames: {:in, [\"server0\", \"server1\", \"server2\"]}]\niex> Server.get_os_by_hostname(params)\n{:ok,\n  [\n    %{hostname: \"server0\", os_name: \"Debian Buster\"},\n    %{hostname: \"server1\", avg_ram: \"Windows 10\"},\n    %{hostname: \"server2\", avg_ram: \"Minix 3\"}\n  ]\n}\n```\n\n#","ref":"readme.html#in-statement"},{"type":"extras","title":"Subqueries and Subfragments - AyeSQL","doc":"Subqueries can be composed directly, as show before, or via the `:inner` tuple\ne.g. let's say we need to get the adults order by name in ascending order and\nage in descending order:\n\n```sql\n-- name: ascending\nASC\n\n-- name: descending\nDESC\n\n-- name: by_age\nage :order_direction\n\n-- name: by_name\nname :order_direction\n\n-- name: get_adults\n-- docs: Gets adults.\nSELECT name, age\n  FROM person\n WHERE age >= 18\nORDER BY :order_by\n```\n\nThen our code in elixir would be:\n\n```elixir\niex> order_by = [\n...>   by_name: [order_direction: :ascending],\n...>   by_age: [order_direction: :descending]\n...> ]\niex> Queries.get_adults(order_by: {:inner, order_by, \", \"})\n{:ok,\n  [\n    %{name: \"Alice\", age: 42},\n    %{name: \"Bob\", age: 21},\n    ...\n  ]\n}\n```\n\n> **Note**: If you're using this level of composability and it fits your use\n> case, consider using either:\n> - [Ecto](https://hexdocs.pm/ecto/Ecto.html)\n> - [EEx templates](https://hexdocs.pm/ayesql/dynamic-queries-with-eex.html)","ref":"readme.html#subqueries-and-subfragments"},{"type":"extras","title":"Installation - AyeSQL","doc":"AyeSQL is available as a Hex package. To install, add it to your\ndependencies in your `mix.exs` file:\n\n```elixir\ndef deps do\n  [{:ayesql, \"~> 1.1\"}]\nend\n```\n\nIf you're going to use any of the provided query runners, then you should add\ntheir dependencies as well:\n\n- Add `:ecto_sql` for `AyeSQL.Runner.Ecto` (default runner).\n- Add `:postgrex` for `AyeSQL.Runner.Postgrex`.\n- Add `:ecto_sql` and `:postgrex` for running queries using `Ecto` in a\n  `PostgreSQL` database.","ref":"readme.html#installation"},{"type":"extras","title":"Author - AyeSQL","doc":"Alexander de Sousa.","ref":"readme.html#author"},{"type":"extras","title":"License - AyeSQL","doc":"AyeSQL is released under the MIT License. See the LICENSE file for further\ndetails.","ref":"readme.html#license"},{"type":"extras","title":"Why Raw SQL","doc":"# Why Raw SQL\n\nWriting and running raw SQL in Elixir is not pretty. Not only the lack of\nsyntax highlighting is horrible, but also substituting parameters into the\nquery string can be unmaintainable e.g:\n\n```elixir\nquery =\n  \"\"\"\n    SELECT hostname, AVG(ram_usage) AS avg_ram\n      FROM server\n     WHERE hostname IN ($1, $2, $3)\n           AND location = $4\n  GROUP BY hostname\n  \"\"\"\narguments = [\"server_0\", \"server_1\", \"server_2\", \"Barcelona\"]\nPostgrex.query(conn, query, arguments)\n```\n\nAdding more hostnames to the previous query would be a nightmare. If\nthe `arguments` are generated dynamically, then editing this query would be\na challenging task.\n\nThankfully, we have [Ecto](https://github.com/elixir-ecto/ecto), which provides\na great DSL for generating database queries at runtime. The same query in Ecto\nwould be the following:\n\n```elixir\nservers = [\"server_0\", \"server_1\", \"server_2\"]\nlocation = \"Barcelona\"\n\nfrom s in \"server\",\n  where: s.location == ^location and s.hostname in ^servers,\n  select: %{hostname: s.hostname, avg_ram: avg(s.ram_usage)}\n```\n\nPretty straightforward and maintainable.\n\nIf Ecto is so good for building queries, **why would you use raw SQL?**. Though\nEcto is quite good with simple queries, complex custom queries often require the\nuse of fragments. Fragments are not pretty though there are workarounds using\nmacros to make them prettier.\n\nIt's easier to see with an example: let's say we have to\n[retrieve the click count of a certain type of link every day of the last N days](https://stackoverflow.com/questions/39556763/use-ecto-to-generate-series-in-postgres-and-also-retrieve-null-values-as-0).\n\nWith complex queries, developers tend to start writing them in raw SQL:\n\n```sql\n    WITH computed_dates AS (\n           SELECT dates::date AS date\n             FROM generate_series(\n                    current_date - $1::interval,\n                    current_date - interval '1 day',\n                    interval '1 day'\n                  ) AS dates\n         )\n  SELECT dates.date AS day, count(clicks.id) AS count\n    FROM computed_dates AS dates\n         LEFT JOIN clicks AS clicks ON date(clicks.inserted_at) = dates.date\n   WHERE clicks.link_id = $2\nGROUP BY dates.date\nORDER BY dates.date;\n```\n\nOnce we have the raw SQL, it's a bit easier to write our Ecto query. In this\ncase, this query should be written using fragments:\n\n```elixir\ndates =\n  \"\"\"\n  SELECT generate_series(\n           current_date - ?::interval,\n           current_date - interval '1 day',\n           interval '1 day'\n         )::date AS d\n  \"\"\"\n\nfrom(\n  c in \"clicks\",\n  right_join: day in fragment(dates, ^days),\n  on: day.d == fragment(\"date(?)\", c.inserted_at),\n  where: c.link_id = ^link_id\n  group_by: day.d,\n  order_by: day.d,\n  select: %{\n    day: fragment(\"date(?)\", day.d),\n    count: count(c.id)\n  }\n)\n```\n\nIt's not the ideal solution (yet) and it's harder to maintain than the raw SQL\nsolution that would work out-of-the-box.\n\nOnly to get to the previous solution, the developer need to:\n\n- Know the specific SQL dialect of the database they're using.\n- Know Ecto's API and its limitations.\n\nFor both, raw SQL and Ecto query, the end result for this query would be the\nsame. With extra effort we found a subpar solution that gives us the same\nresult as our raw SQL.\n\nThe final, and sometimes, optional step would be to transform the Ecto query\ninto something a bit more maintainable.\n\n```elixir\ndefmodule CustomDSL do\n  defmacro date(date) do\n    quote do\n      fragment(\"date(?)\", unquote(date))\n    end\n  end\n\n  defmacro ndays(n) do\n    query =\n      \"\"\"\n      SELECT generate_series(\n               current_date - ?::interval,\n               current_date - interval '1 day',\n               interval '1 day'\n             )::date AS d\n      \"\"\"\n\n    quote do\n      fragment(unquote(query), unquote(n))\n    end\n  end\nend\n\nimport CustomDSL\n\nfrom(\n  c in \"clicks\",\n  right_join: day in ndays(^days),\n  on: day.d == date(c.inserted_at),\n  where: c.link_id = ^link_id\n  group_by: day.d,\n  order_by: day.d,\n  select: %{\n    day: date(day.d)\n    count: count(c.id)\n  }\n)\n```\n\nThe previous query is more readable, but requires knowledge of:\n\n- The specific SQL dialect.\n- Ecto's API and its limitations.\n- Elixir's macros.\n- Custom DSL API.\n\nFor some problems, getting to this final stage is preferable. However, for\nsome other problems, the raw SQL query would have been enough.\n\nThe raw SQL query was already a good solution to the problem. It only needs a\nmaintainable way to be parameterized. That's why AyeSQL exists.","ref":"why-raw-sql.html"},{"type":"extras","title":"Debugging Queries","doc":"# Debugging Queries\n\nQueries are run by default. To avoid this and get the `AyeSQL.Query.t()` struct\ninstead, set the option `run: false` on the top of the module e.g:\n\n```elixir\ndefmodule Queries do\n  use AyeSQL, repo: MyRepo, run: false\n\n  defqueries(\"myqueries.sql\")\nend\n```\n\nor when running a query:\n\n```elixir\niex> params = [\n...>   link_id: 42,\n...>   days: %Postgrex.Interval{secs: 864_000} # 10 days\n...> ]\niex> Queries.get_avg_clicks(params, run: false)\n{:ok, %AyeSQL.Query{statement: ..., arguments: ...}}\n```","ref":"debugging-queries.html"},{"type":"extras","title":"Dynamic Queries With EEx","doc":"# Dynamic Queries With EEx\n\nWhen AyeSQL syntax for dynamic query generation falls short, we can leverage\n`EEx` templates for generating them.\n\n> Note: It's not possible to use `defqueries/1` and `defqueries/3` macros when\n> using `EEx` templates. We'll need to use the function `AyeSQL.eval_query/2`\n> instead. This function will generate an anonymous function with the query.\n\nOne of such cases is when the identifiers in the query are not constant (table\nnames, aliases, etc). The following `EEx` template has a query with dynamic\ntables, aliases, fragments and even joins:\n\n```sql\n-- File: queries.sql.eex\n \nSELECT\n   .datetime AS datetime,\n    AS calculation\nFROM\n    AS  \n   \n  INNER JOIN   AS  \n    ON  .datetime =  .datetime\n   \nWHERE\n   .datetime BETWEEN :start_date AND :end_date\n```\n\nWith the previous template, we can generate a valid AyeSQL query with the\nfollowing:\n\n```elixir\niex> assigns = [\n...>   calculation: \"(a.value / b.value)\",\n...>   tables: [\n...>     %{name: \"table_a\", as: \"a\"},\n...>     %{name: \"table_b\", as: \"b\"}\n...>    ]\n...> ]\niex> dynamic_query =\n...>   \"queries.sql.eex\"\n...>   |> File.read!()\n...>   |> EEx.eval_string(assigns: assigns)\n```\n\nso, the variable `dynamic_query` contains the following query:\n\n```sql\nSELECT\n  a.datetime AS datetime,\n  (a.value / b.value) AS calculation\nFROM\n  table_a AS a\n  INNER JOIN table_b AS b\n    ON b.datetime = a.datetime\nWHERE\n  a.datetime BETWEEN :start_date AND :end_date\n```\n\nThis query is then compatible with `AyeSQL`, so we can compile it into a\nfunction using the following:\n\n```elixir\niex> function = AyeSQL.eval_query(dynamic_query)\n```\n\nAnd finally use the function as any other `AyeSQL` function:\n\n```elixir\niex> {:ok, start_date, _} = DateTime.from_iso8601(\"2022-04-17T00:00:00Z\")\niex> function.([start_date: start_date, end_date: DateTime.utc_now()], repo: MyApp.Repo)\n{:ok, [%{datetime: ..., calculation: ...}, ...]}\n```","ref":"dynamic-queries-with-eex.html"},{"type":"extras","title":"Query Runners","doc":"# Query Runners\n\n`AyeSQL` can be extended to support more databases than PostgreSQL by implementing\nthe behaviour `AyeSQL.Runner`. This library ships with two runners already:\n\n- `AyeSQL.Runner.Ecto`, (it's the default and assumes PostgreSQL is the database).\n- `AyeSQL.Runner.Postgrex`.\n\nBy default, AyeSQL uses the runner `AyeSQL.Runner.Ecto`. This runner only has\none option which is `:repo` for the repo module. Additionally, it converts the\nresult to a list of maps.\n\nIn order to support your database, you just need to implement the behaviour\ne.g. let's say we implement a `MySQL` our queries module would be created using\nsomething like the following:\n\n```elixir\ndefmodule MyApp.Queries do\n  use AyeSQL, runner: MyApp.Runner.MySQL, pool: MyApp.MySQL.Pool\n\n  defqueries(\"queries.sql\")\nend\n```\n\nor\n\n```elixir\nimport AyeSQL, only: [defqueries: 3]\n\ndefqueries(Queries, \"queries.sql\",\n  runner: MyApp.Runner.MySQL,\n  pool: MyApp.MySQL.Pool\n)\n```\n\nAnd our runner could be implemented like this:\n\n```elixir\ndefmodule MyApp.Runner.MySQL do\n  use AyeSQL.Runner\n\n  @impl AyeSQL\n  def run(%AyeSQL.Query{statement: stmt, arguments: args}, options) do\n    query_options = Keyword.frop(options, [:pool, :into])\n    stmt = transform_stmt(stmt)\n\n    with {:ok, result} <- MyXQL.query(pool, stmt, args, query_options) do\n      result = AyeSQL.Runner.handle_result(result)\n      {:ok, result}\n    end\n  end\n\n  @spec transform_stmt(AyeSQL.Query.statement()) :: AyeSQL.Query.statement()\n  defp transform_stmt(stmt) do\n    Regex.replace(~r/\\$(\\d+)/, stmt, \"?\")\n  end\nend\n```","ref":"query-runners.html"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v1.1.2 - Changelog","doc":"#","ref":"changelog.html#v1-1-2"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Updated dependencies.\n  * Fixed CI.\n  * Runners are now more flexible and can return the raw `Postgrex` result or\n    inside maps, keyword list and custom structs.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v1.1.0 - Changelog","doc":"#","ref":"changelog.html#v1-1-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added new lexer with improved tokens. Now these tokens track:\n    + lines and columns for each token.\n    + original and modified contents.\n  * Added new parser with support for both named and unamed queries.\n  * Improved compiler errors. Now they show:\n    + line and columns where errors occur.\n    + surrounding lines containing the error.\n  * Added missing tests for the runners.\n  * Reorganized documentation.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v1.0.0 - Changelog","doc":"#","ref":"changelog.html#v1-0-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added support for specific runner options e.g. adding a `timeout` to the\n    `Postgrex` query.\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* Now it runs queries by default. To avoid running the queries, add the\n    option `run: false` on the module or the function calls.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.6.1 - Changelog","doc":"#","ref":"changelog.html#v0-6-1"},{"type":"extras","title":"Bug fix - Changelog","doc":"* [Dax Ravi](https://github.com/thdxr) fixed an infinite recursion problem\n    in fragment token regular expression in the lexer.","ref":"changelog.html#bug-fix"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"#","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added support for subqueries with local scope.\n  * Improved documentation.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.5.5 - Changelog","doc":"#","ref":"changelog.html#v0-5-5"},{"type":"extras","title":"Bug fix - Changelog","doc":"* Added missing support for function calls by name in parameters.","ref":"changelog.html#bug-fix"},{"type":"extras","title":"v0.5.4 - Changelog","doc":"#","ref":"changelog.html#v0-5-4"},{"type":"extras","title":"Bug fix - Changelog","doc":"* Added support for `nil` values for parameters (`NULL`).","ref":"changelog.html#bug-fix"},{"type":"extras","title":"v0.5.3 - Changelog","doc":"#","ref":"changelog.html#v0-5-3"},{"type":"extras","title":"Bug fix - Changelog","doc":"* Added support for Windows new line.","ref":"changelog.html#bug-fix"},{"type":"extras","title":"v0.5.2 - Changelog","doc":"#","ref":"changelog.html#v0-5-2"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Fixed dialyzer warnings for unexistent type `AyeSQL.Core.query()`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.5.1 - Changelog","doc":"#","ref":"changelog.html#v0-5-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Improved documentation.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"This version maintains the same query language, but it breaks runners as now\nqueries return `AyeSQL.Query.t()` instead of `{binary(), [term()]}`.\n\n#","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Refactored code to improve readability.\n  * Added `AyeSQL.Query` for queries (instead of tuple).\n  * Added `AyeSQL.Error` for query errors (instead of returning a string with\n    the missing parameter).\n  * Added `AyeSQL.AST.Context` to be able to handle several errors, instead of\n    returning them one at the time.\n  * Improved documentation explaining all features.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.4.1 - Changelog","doc":"#","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added optional arguments for better composition.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"#","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added `AyeSQL.Runner` behaviour for writing custom query runners.\n  * New query runners for Ecto and Postgrex connections.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.3.2 - Changelog","doc":"* Added configuration support for running queries by default.","ref":"changelog.html#v0-3-2"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"#","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added `defqueries/3` to avoid boiler plate code.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"#","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Simplified lexer and parser.\n  * Now it's possible to accept anonymous blocks of code.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"#","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Now it detects when an SQL file has been changed (suggested by\n    [Ole Morten Halvorsen](https://github.com/omh)).\n  * Updated dependencies.\n  * Improved code for maintainability.\n  * Improved tests for more code coverage.\n  * Support for Elixir 1.8 and Erlang 21","ref":"changelog.html#enhancements"}],"content_type":"text/markdown"}